@@ -2,7 +2,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from calendar-common-sets.js */
+/* import-globals-from calendar-command-controller.js */
 
 /* globals MozXULElement CalendarTaskTreeView */
 
@@ -122,7 +122,7 @@
                      itemproperty="completed"
                      label="&calendar.unifinder.tree.done.label;"
                      tooltiptext="&calendar.unifinder.tree.done.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="2"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 2;"/>
             <treecol is="treecol-image" id="calendar-task-tree-col-priority"
                      class="calendar-task-tree-col-priority"
                      minwidth="17"
@@ -130,63 +130,63 @@
                      itemproperty="priority"
                      label="&calendar.unifinder.tree.priority.label;"
                      tooltiptext="&calendar.unifinder.tree.priority.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="4"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 4;"/>
             <treecol class="calendar-task-tree-col-title"
                      itemproperty="title"
                      flex="1"
                      label="&calendar.unifinder.tree.title.label;"
                      tooltiptext="&calendar.unifinder.tree.title.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="6"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 6;"/>
             <treecol class="calendar-task-tree-col-entrydate"
                      itemproperty="entryDate"
                      flex="1"
                      label="&calendar.unifinder.tree.startdate.label;"
                      tooltiptext="&calendar.unifinder.tree.startdate.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="8"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 8;"/>
             <treecol class="calendar-task-tree-col-duedate"
                      itemproperty="dueDate"
                      flex="1"
                      label="&calendar.unifinder.tree.duedate.label;"
                      tooltiptext="&calendar.unifinder.tree.duedate.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="10"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 10;"/>
             <treecol class="calendar-task-tree-col-duration"
                      itemproperty="duration"
                      sortKey="dueDate"
                      flex="1"
                      label="&calendar.unifinder.tree.duration.label;"
                      tooltiptext="&calendar.unifinder.tree.duration.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="12"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 12;"/>
             <treecol class="calendar-task-tree-col-completeddate"
                      itemproperty="completedDate"
                      flex="1"
                      label="&calendar.unifinder.tree.completeddate.label;"
                      tooltiptext="&calendar.unifinder.tree.completeddate.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="14"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 14;"/>
             <treecol class="calendar-task-tree-col-percentcomplete"
                      itemproperty="percentComplete"
                      flex="1"
                      minwidth="40"
                      label="&calendar.unifinder.tree.percentcomplete.label;"
                      tooltiptext="&calendar.unifinder.tree.percentcomplete.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="16"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 16;"/>
             <treecol class="calendar-task-tree-col-categories"
                      itemproperty="categories"
                      flex="1"
                      label="&calendar.unifinder.tree.categories.label;"
                      tooltiptext="&calendar.unifinder.tree.categories.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="18"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 18;"/>
             <treecol class="calendar-task-tree-col-location"
                      itemproperty="location"
                      flex="1"
                      label="&calendar.unifinder.tree.location.label;"
                      tooltiptext="&calendar.unifinder.tree.location.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="20"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 20;"/>
             <treecol class="calendar-task-tree-col-status"
                      itemproperty="status"
                      flex="1"
                      label="&calendar.unifinder.tree.status.label;"
                      tooltiptext="&calendar.unifinder.tree.status.tooltip2;"/>
-            <splitter class="tree-splitter" ordinal="22"/>
+            <splitter class="tree-splitter" style="-moz-box-ordinal-group: 22;"/>
             <treecol class="calendar-task-tree-col-calendar"
                      itemproperty="calendar"
                      flex="1"
@@ -373,7 +373,7 @@
           col.setAttribute("hidden", "true");
         }
         if (ordinals && ordinals.length > 0) {
-          col.ordinal = Number(ordinals.shift());
+          col.ordinal = ordinals.shift();
         }
         if (widths && widths.length > 0) {
           col.width = Number(widths.shift());
@@ -424,10 +424,9 @@
           prefix +
           PluralForm.get(dur.hours, cal.l10n.getCalString("dueInHours")).replace("#1", dur.hours)
         );
-      } else {
-        // Less than one hour.
-        return cal.l10n.getCalString("dueInLessThanOneHour");
       }
+      // Less than one hour.
+      return cal.l10n.getCalString("dueInLessThanOneHour");
     }
 
     /**
@@ -459,15 +458,21 @@
         operation: null,
 
         onOperationComplete(opCalendar, status, operationType, id, dateTime) {
+          if (!this.tree.mTreeView.tree) {
+            // Looks like we've been disconnected from the DOM, there's no point in continuing.
+            return;
+          }
+
           if (opCalendar.id in this.tree.mPendingRefreshJobs) {
             delete this.tree.mPendingRefreshJobs[opCalendar.id];
           }
 
           let oldItems = this.tree.mTaskArray.filter(item => item.calendar.id == opCalendar.id);
           this.tree.mTreeView.modifyItems(this.items, oldItems);
+          this.tree.dispatchEvent(new CustomEvent("refresh", { bubbles: false }));
         },
 
-        onGetResult(opCalendar, status, itemType, detail, count, items) {
+        onGetResult(opCalendar, status, itemType, detail, items) {
           this.items = this.items.concat(items);
         },
 
@@ -518,7 +523,7 @@
 
       Services.prefs.getBranch("").addObserver("calendar.", this.mPrefObserver);
 
-      const cals = cal.view.getCompositeCalendar(window).getCalendars({}) || [];
+      const cals = cal.view.getCompositeCalendar(window).getCalendars() || [];
       const enabledCals = cals.filter(calendar => !calendar.getProperty("disabled"));
 
       enabledCals.forEach(calendar => this.refreshFromCalendar(calendar));
