@@ -3,7 +3,7 @@
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var irc = {};
-Services.scriptloader.loadSubScript("resource:///components/irc.js", irc);
+Services.scriptloader.loadSubScript("resource:///modules/irc.jsm", irc);
 
 var messages = {
   // Exactly 51 characters.
@@ -30,7 +30,7 @@ irc.GenericIRCConversation._account = {
 function run_test() {
   for (let message in messages) {
     let msg = { message };
-    let generatedMsgs = irc.GenericIRCConversation.prepareForSending(msg, {});
+    let generatedMsgs = irc.GenericIRCConversation.prepareForSending(msg);
 
     // The expected messages as defined above.
     let expectedMsgs = messages[message];
