@@ -4,9 +4,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsAutoPtr.h"
 #include "nsIDocShell.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsIDocumentLoader.h"
 #include "nsIObserverService.h"
 #include "nsIXULRuntime.h"
@@ -15,8 +14,10 @@
 #include "RequestContextService.h"
 
 #include "mozilla/Atomics.h"
+#include "mozilla/ClearOnShutdown.h"
 #include "mozilla/Logging.h"
 #include "mozilla/Services.h"
+#include "mozilla/StaticPtr.h"
 #include "mozilla/TimeStamp.h"
 
 #include "mozilla/net/NeckoChild.h"
@@ -32,20 +33,21 @@ LazyLogModule gRequestContextLog("RequestContext");
 #undef LOG
 #define LOG(args) MOZ_LOG(gRequestContextLog, LogLevel::Info, args)
 
+static StaticRefPtr<RequestContextService> gSingleton;
+
 // This is used to prevent adding tail pending requests after shutdown
 static bool sShutdown = false;
 
 // nsIRequestContext
-class RequestContext final : public nsIRequestContext
-                           , public nsITimerCallback
-{
-public:
+class RequestContext final : public nsIRequestContext, public nsITimerCallback {
+ public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIREQUESTCONTEXT
   NS_DECL_NSITIMERCALLBACK
 
   explicit RequestContext(const uint64_t id);
-private:
+
+ private:
   virtual ~RequestContext();
 
   void ProcessTailQueue(nsresult aResult);
@@ -55,9 +57,8 @@ private:
   void RescheduleUntailTimer(TimeStamp const& now);
 
   uint64_t mID;
-  Atomic<uint32_t>       mBlockingTransactionCount;
-  nsAutoPtr<SpdyPushCache> mSpdyCache;
-  nsCString mUserAgentOverride;
+  Atomic<uint32_t> mBlockingTransactionCount;
+  UniquePtr<SpdyPushCache> mSpdyCache;
 
   typedef nsCOMPtr<nsIRequestTailUnblockCallback> PendingTailRequest;
   // Number of known opened non-tailed requets
@@ -91,25 +92,22 @@ private:
 NS_IMPL_ISUPPORTS(RequestContext, nsIRequestContext, nsITimerCallback)
 
 RequestContext::RequestContext(const uint64_t aID)
-  : mID(aID)
-  , mBlockingTransactionCount(0)
-  , mNonTailRequests(0)
-  , mAfterDOMContentLoaded(false)
-{
+    : mID(aID),
+      mBlockingTransactionCount(0),
+      mNonTailRequests(0),
+      mAfterDOMContentLoaded(false) {
   LOG(("RequestContext::RequestContext this=%p id=%" PRIx64, this, mID));
 }
 
-RequestContext::~RequestContext()
-{
+RequestContext::~RequestContext() {
   MOZ_ASSERT(mTailQueue.Length() == 0);
 
-  LOG(("RequestContext::~RequestContext this=%p blockers=%u",
-       this, static_cast<uint32_t>(mBlockingTransactionCount)));
+  LOG(("RequestContext::~RequestContext this=%p blockers=%u", this,
+       static_cast<uint32_t>(mBlockingTransactionCount)));
 }
 
 NS_IMETHODIMP
-RequestContext::BeginLoad()
-{
+RequestContext::BeginLoad() {
   MOZ_ASSERT(NS_IsMainThread());
 
   LOG(("RequestContext::BeginLoad %p", this));
@@ -128,8 +126,7 @@ RequestContext::BeginLoad()
 }
 
 NS_IMETHODIMP
-RequestContext::DOMContentLoaded()
-{
+RequestContext::DOMContentLoaded() {
   MOZ_ASSERT(NS_IsMainThread());
 
   LOG(("RequestContext::DOMContentLoaded %p", this));
@@ -155,90 +152,58 @@ RequestContext::DOMContentLoaded()
 }
 
 NS_IMETHODIMP
-RequestContext::GetBlockingTransactionCount(uint32_t *aBlockingTransactionCount)
-{
+RequestContext::GetBlockingTransactionCount(
+    uint32_t* aBlockingTransactionCount) {
   NS_ENSURE_ARG_POINTER(aBlockingTransactionCount);
   *aBlockingTransactionCount = mBlockingTransactionCount;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-RequestContext::AddBlockingTransaction()
-{
+RequestContext::AddBlockingTransaction() {
   mBlockingTransactionCount++;
-  LOG(("RequestContext::AddBlockingTransaction this=%p blockers=%u",
-       this, static_cast<uint32_t>(mBlockingTransactionCount)));
+  LOG(("RequestContext::AddBlockingTransaction this=%p blockers=%u", this,
+       static_cast<uint32_t>(mBlockingTransactionCount)));
   return NS_OK;
 }
 
 NS_IMETHODIMP
-RequestContext::RemoveBlockingTransaction(uint32_t *outval)
-{
+RequestContext::RemoveBlockingTransaction(uint32_t* outval) {
   NS_ENSURE_ARG_POINTER(outval);
   mBlockingTransactionCount--;
-  LOG(("RequestContext::RemoveBlockingTransaction this=%p blockers=%u",
-       this, static_cast<uint32_t>(mBlockingTransactionCount)));
+  LOG(("RequestContext::RemoveBlockingTransaction this=%p blockers=%u", this,
+       static_cast<uint32_t>(mBlockingTransactionCount)));
   *outval = mBlockingTransactionCount;
   return NS_OK;
 }
 
-NS_IMETHODIMP
-RequestContext::GetSpdyPushCache(mozilla::net::SpdyPushCache **aSpdyPushCache)
-{
-  *aSpdyPushCache = mSpdyCache.get();
-  return NS_OK;
+SpdyPushCache* RequestContext::GetSpdyPushCache() { return mSpdyCache.get(); }
+
+void RequestContext::SetSpdyPushCache(SpdyPushCache* aSpdyPushCache) {
+  mSpdyCache = WrapUnique(aSpdyPushCache);
 }
 
-NS_IMETHODIMP
-RequestContext::SetSpdyPushCache(mozilla::net::SpdyPushCache *aSpdyPushCache)
-{
-  mSpdyCache = aSpdyPushCache;
-  return NS_OK;
-}
+uint64_t RequestContext::GetID() { return mID; }
 
 NS_IMETHODIMP
-RequestContext::GetID(uint64_t *outval)
-{
-  NS_ENSURE_ARG_POINTER(outval);
-  *outval = mID;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-RequestContext::GetUserAgentOverride(nsACString& aUserAgentOverride)
-{
-  aUserAgentOverride = mUserAgentOverride;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-RequestContext::SetUserAgentOverride(const nsACString& aUserAgentOverride)
-{
-  mUserAgentOverride = aUserAgentOverride;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-RequestContext::AddNonTailRequest()
-{
+RequestContext::AddNonTailRequest() {
   MOZ_ASSERT(NS_IsMainThread());
 
   ++mNonTailRequests;
-  LOG(("RequestContext::AddNonTailRequest this=%p, cnt=%u",
-       this, mNonTailRequests));
+  LOG(("RequestContext::AddNonTailRequest this=%p, cnt=%u", this,
+       mNonTailRequests));
 
   ScheduleUnblock();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-RequestContext::RemoveNonTailRequest()
-{
+RequestContext::RemoveNonTailRequest() {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(mNonTailRequests > 0);
 
-  LOG(("RequestContext::RemoveNonTailRequest this=%p, cnt=%u",
-       this, mNonTailRequests - 1));
+  LOG(("RequestContext::RemoveNonTailRequest this=%p, cnt=%u", this,
+       mNonTailRequests - 1));
 
   --mNonTailRequests;
 
@@ -246,9 +211,7 @@ RequestContext::RemoveNonTailRequest()
   return NS_OK;
 }
 
-void
-RequestContext::ScheduleUnblock()
-{
+void RequestContext::ScheduleUnblock() {
   MOZ_ASSERT(!IsNeckoChild());
   MOZ_ASSERT(NS_IsMainThread());
 
@@ -256,21 +219,23 @@ RequestContext::ScheduleUnblock()
     return;
   }
 
-  uint32_t quantum = gHttpHandler->TailBlockingDelayQuantum(mAfterDOMContentLoaded);
+  uint32_t quantum =
+      gHttpHandler->TailBlockingDelayQuantum(mAfterDOMContentLoaded);
   uint32_t delayMax = gHttpHandler->TailBlockingDelayMax();
   uint32_t totalMax = gHttpHandler->TailBlockingTotalMax();
 
   if (!mBeginLoadTime.IsNull()) {
-    // We decrease the maximum delay progressively with the time since the page load
-    // begin.  This seems like a reasonable and clear heuristic allowing us to start
-    // loading tailed requests in a deterministic time after the load has started.
+    // We decrease the maximum delay progressively with the time since the page
+    // load begin.  This seems like a reasonable and clear heuristic allowing us
+    // to start loading tailed requests in a deterministic time after the load
+    // has started.
 
     uint32_t sinceBeginLoad = static_cast<uint32_t>(
-      (TimeStamp::NowLoRes() - mBeginLoadTime).ToMilliseconds());
+        (TimeStamp::NowLoRes() - mBeginLoadTime).ToMilliseconds());
     uint32_t tillTotal = totalMax - std::min(sinceBeginLoad, totalMax);
-    uint32_t proportion = totalMax // values clamped between 0 and 60'000
-      ? (delayMax * tillTotal) / totalMax
-      : 0;
+    uint32_t proportion = totalMax  // values clamped between 0 and 60'000
+                              ? (delayMax * tillTotal) / totalMax
+                              : 0;
     delayMax = std::min(delayMax, proportion);
   }
 
@@ -287,8 +252,11 @@ RequestContext::ScheduleUnblock()
     delay = delayMax;
   }
 
-  LOG(("RequestContext::ScheduleUnblock this=%p non-tails=%u tail-queue=%zu delay=%u after-DCL=%d",
-       this, mNonTailRequests, mTailQueue.Length(), delay.value(), mAfterDOMContentLoaded));
+  LOG(
+      ("RequestContext::ScheduleUnblock this=%p non-tails=%u tail-queue=%zu "
+       "delay=%u after-DCL=%d",
+       this, mNonTailRequests, mTailQueue.Length(), delay.value(),
+       mAfterDOMContentLoaded));
 
   TimeStamp now = TimeStamp::NowLoRes();
   mUntailAt = now + TimeDuration::FromMilliseconds(delay.value());
@@ -299,9 +267,7 @@ RequestContext::ScheduleUnblock()
   }
 }
 
-void
-RequestContext::RescheduleUntailTimer(TimeStamp const& now)
-{
+void RequestContext::RescheduleUntailTimer(TimeStamp const& now) {
   MOZ_ASSERT(mUntailAt >= now);
 
   if (mUntailTimer) {
@@ -334,8 +300,7 @@ RequestContext::RescheduleUntailTimer(TimeStamp const& now)
 }
 
 NS_IMETHODIMP
-RequestContext::Notify(nsITimer *timer)
-{
+RequestContext::Notify(nsITimer* timer) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(timer == mUntailTimer);
   MOZ_ASSERT(!mTimerScheduledAt.IsNull());
@@ -359,9 +324,8 @@ RequestContext::Notify(nsITimer *timer)
 }
 
 NS_IMETHODIMP
-RequestContext::IsContextTailBlocked(nsIRequestTailUnblockCallback * aRequest,
-                                     bool *aBlocked)
-{
+RequestContext::IsContextTailBlocked(nsIRequestTailUnblockCallback* aRequest,
+                                     bool* aBlocked) {
   MOZ_ASSERT(NS_IsMainThread());
 
   LOG(("RequestContext::IsContextTailBlocked this=%p, request=%p, queued=%zu",
@@ -402,14 +366,13 @@ RequestContext::IsContextTailBlocked(nsIRequestTailUnblockCallback * aRequest,
 }
 
 NS_IMETHODIMP
-RequestContext::CancelTailedRequest(nsIRequestTailUnblockCallback * aRequest)
-{
+RequestContext::CancelTailedRequest(nsIRequestTailUnblockCallback* aRequest) {
   MOZ_ASSERT(NS_IsMainThread());
 
   bool removed = mTailQueue.RemoveElement(aRequest);
 
-  LOG(("RequestContext::CancelTailedRequest %p req=%p removed=%d",
-       this, aRequest, removed));
+  LOG(("RequestContext::CancelTailedRequest %p req=%p removed=%d", this,
+       aRequest, removed));
 
   // Stop untail timer if all tail requests are canceled.
   if (removed && mTailQueue.IsEmpty()) {
@@ -425,9 +388,7 @@ RequestContext::CancelTailedRequest(nsIRequestTailUnblockCallback * aRequest)
   return NS_OK;
 }
 
-void
-RequestContext::ProcessTailQueue(nsresult aResult)
-{
+void RequestContext::ProcessTailQueue(nsresult aResult) {
   LOG(("RequestContext::ProcessTailQueue this=%p, queued=%zu, rv=%" PRIx32,
        this, mTailQueue.Length(), static_cast<uint32_t>(aResult)));
 
@@ -442,15 +403,14 @@ RequestContext::ProcessTailQueue(nsresult aResult)
   nsTArray<PendingTailRequest> queue;
   queue.SwapElements(mTailQueue);
 
-  for (auto request : queue) {
+  for (const auto& request : queue) {
     LOG(("  untailing %p", request.get()));
     request->OnTailUnblock(aResult);
   }
 }
 
 NS_IMETHODIMP
-RequestContext::CancelTailPendingRequests(nsresult aResult)
-{
+RequestContext::CancelTailPendingRequests(nsresult aResult) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(NS_FAILED(aResult));
 
@@ -458,14 +418,13 @@ RequestContext::CancelTailPendingRequests(nsresult aResult)
   return NS_OK;
 }
 
-//nsIRequestContextService
-RequestContextService *RequestContextService::sSelf = nullptr;
+// nsIRequestContextService
+RequestContextService* RequestContextService::sSelf = nullptr;
 
 NS_IMPL_ISUPPORTS(RequestContextService, nsIRequestContextService, nsIObserver)
 
 RequestContextService::RequestContextService()
-  : mNextRCID(1)
-{
+    : mRCIDNamespace(0), mNextRCID(1) {
   MOZ_ASSERT(!sSelf, "multiple rcs instances!");
   MOZ_ASSERT(NS_IsMainThread());
   sSelf = this;
@@ -474,16 +433,13 @@ RequestContextService::RequestContextService()
   runtime->GetProcessID(&mRCIDNamespace);
 }
 
-RequestContextService::~RequestContextService()
-{
+RequestContextService::~RequestContextService() {
   MOZ_ASSERT(NS_IsMainThread());
   Shutdown();
   sSelf = nullptr;
 }
 
-nsresult
-RequestContextService::Init()
-{
+nsresult RequestContextService::Init() {
   nsresult rv;
 
   MOZ_ASSERT(NS_IsMainThread());
@@ -504,9 +460,7 @@ RequestContextService::Init()
   return NS_OK;
 }
 
-void
-RequestContextService::Shutdown()
-{
+void RequestContextService::Shutdown() {
   MOZ_ASSERT(NS_IsMainThread());
   // We need to do this to prevent the requests from being scheduled after
   // shutdown.
@@ -517,24 +471,32 @@ RequestContextService::Shutdown()
   sShutdown = true;
 }
 
-/* static */ nsresult
-RequestContextService::Create(nsISupports *aOuter, const nsIID& aIID, void **aResult)
-{
+/* static */
+already_AddRefed<nsIRequestContextService>
+RequestContextService::GetOrCreate() {
   MOZ_ASSERT(NS_IsMainThread());
-  if (aOuter != nullptr) {
-    return NS_ERROR_NO_AGGREGATION;
+
+  if (sShutdown) {
+    return nullptr;
   }
 
-  RefPtr<RequestContextService> svc = new RequestContextService();
-  nsresult rv = svc->Init();
-  NS_ENSURE_SUCCESS(rv, rv);
+  RefPtr<RequestContextService> svc;
+  if (gSingleton) {
+    svc = gSingleton;
+  } else {
+    svc = new RequestContextService();
+    nsresult rv = svc->Init();
+    NS_ENSURE_SUCCESS(rv, nullptr);
+    gSingleton = svc;
+    ClearOnShutdown(&gSingleton);
+  }
 
-  return svc->QueryInterface(aIID, aResult);
+  return svc.forget();
 }
 
 NS_IMETHODIMP
-RequestContextService::GetRequestContext(const uint64_t rcID, nsIRequestContext **rc)
-{
+RequestContextService::GetRequestContext(const uint64_t rcID,
+                                         nsIRequestContext** rc) {
   MOZ_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG_POINTER(rc);
   *rc = nullptr;
@@ -543,6 +505,10 @@ RequestContextService::GetRequestContext(const uint64_t rcID, nsIRequestContext
     return NS_ERROR_ILLEGAL_DURING_SHUTDOWN;
   }
 
+  if (!rcID) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
   if (!mTable.Get(rcID, rc)) {
     nsCOMPtr<nsIRequestContext> newSC = new RequestContext(rcID);
     mTable.Put(rcID, newSC);
@@ -553,8 +519,8 @@ RequestContextService::GetRequestContext(const uint64_t rcID, nsIRequestContext
 }
 
 NS_IMETHODIMP
-RequestContextService::GetRequestContextFromLoadGroup(nsILoadGroup *aLoadGroup, nsIRequestContext **rc)
-{
+RequestContextService::GetRequestContextFromLoadGroup(nsILoadGroup* aLoadGroup,
+                                                      nsIRequestContext** rc) {
   nsresult rv;
 
   uint64_t rcID;
@@ -567,8 +533,7 @@ RequestContextService::GetRequestContextFromLoadGroup(nsILoadGroup *aLoadGroup,
 }
 
 NS_IMETHODIMP
-RequestContextService::NewRequestContext(nsIRequestContext **rc)
-{
+RequestContextService::NewRequestContext(nsIRequestContext** rc) {
   MOZ_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG_POINTER(rc);
   *rc = nullptr;
@@ -577,7 +542,9 @@ RequestContextService::NewRequestContext(nsIRequestContext **rc)
     return NS_ERROR_ILLEGAL_DURING_SHUTDOWN;
   }
 
-  uint64_t rcID = ((static_cast<uint64_t>(mRCIDNamespace) << 32) & 0xFFFFFFFF00000000LL) | mNextRCID++;
+  uint64_t rcID =
+      ((static_cast<uint64_t>(mRCIDNamespace) << 32) & 0xFFFFFFFF00000000LL) |
+      mNextRCID++;
 
   nsCOMPtr<nsIRequestContext> newSC = new RequestContext(rcID);
   mTable.Put(rcID, newSC);
@@ -587,21 +554,15 @@ RequestContextService::NewRequestContext(nsIRequestContext **rc)
 }
 
 NS_IMETHODIMP
-RequestContextService::RemoveRequestContext(const uint64_t rcID)
-{
-  if (IsNeckoChild() && gNeckoChild) {
-    gNeckoChild->SendRemoveRequestContext(rcID);
-  }
-
+RequestContextService::RemoveRequestContext(const uint64_t rcID) {
   MOZ_ASSERT(NS_IsMainThread());
   mTable.Remove(rcID);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-RequestContextService::Observe(nsISupports *subject, const char *topic,
-                                  const char16_t *data_unicode)
-{
+RequestContextService::Observe(nsISupports* subject, const char* topic,
+                               const char16_t* data_unicode) {
   MOZ_ASSERT(NS_IsMainThread());
   if (!strcmp(NS_XPCOM_SHUTDOWN_OBSERVER_ID, topic)) {
     Shutdown();
@@ -609,7 +570,7 @@ RequestContextService::Observe(nsISupports *subject, const char *topic,
   }
 
   if (!strcmp("content-document-interactive", topic)) {
-    nsCOMPtr<nsIDocument> document(do_QueryInterface(subject));
+    nsCOMPtr<dom::Document> document(do_QueryInterface(subject));
     MOZ_ASSERT(document);
     // We want this be triggered also for iframes, since those track their
     // own request context ids.
@@ -643,7 +604,7 @@ RequestContextService::Observe(nsISupports *subject, const char *topic,
   return NS_OK;
 }
 
-} // ::mozilla::net
-} // ::mozilla
+}  // namespace net
+}  // namespace mozilla
 
 #undef LOG
