@@ -5,35 +5,34 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "GetAddrInfo.h"
+#include "mozilla/ClearOnShutdown.h"
 #include "mozilla/net/DNS.h"
 #include "prnetdb.h"
 #include "nsHostResolver.h"
 #include "nsError.h"
-#include "mozilla/Mutex.h"
-#include "nsAutoPtr.h"
-#include "mozilla/StaticPtr.h"
-#include "MainThreadUtils.h"
-#include "mozilla/DebugOnly.h"
 #include "mozilla/net/DNS.h"
 #include <algorithm>
 #include "prerror.h"
 
 #include "mozilla/Logging.h"
+#include "mozilla/StaticPrefs_network.h"
 
 #ifdef DNSQUERY_AVAILABLE
 // There is a bug in windns.h where the type of parameter ppQueryResultsSet for
 // DnsQuery_A is dependent on UNICODE being set. It should *always* be
 // PDNS_RECORDA, but if UNICODE is set it is PDNS_RECORDW. To get around this
 // we make sure that UNICODE is unset.
-#undef UNICODE
-#include <ws2tcpip.h>
-#undef GetAddrInfo
-#include <windns.h>
+#  undef UNICODE
+#  include <ws2tcpip.h>
+#  undef GetAddrInfo
+#  include <windns.h>
 #endif
 
 namespace mozilla {
 namespace net {
 
+static StaticRefPtr<NativeDNSResolverOverride> gOverrideService;
+
 static LazyLogModule gGetAddrInfoLog("GetAddrInfo");
 #define LOG(msg, ...) \
   MOZ_LOG(gGetAddrInfoLog, LogLevel::Debug, ("[DNS]: " msg, ##__VA_ARGS__))
@@ -51,109 +50,17 @@ static_assert(PR_AF_INET == AF_INET && PR_AF_INET6 == AF_INET6 &&
                   PR_AF_UNSPEC == AF_UNSPEC,
               "PR_AF_* must match AF_*");
 
-// We intentionally leak this mutex. This is because we can run into a
-// situation where the worker threads are still running until the process
-// is actually fully shut down, and at any time one of those worker
-// threads can access gDnsapiInfoLock.
-static OffTheBooksMutex* gDnsapiInfoLock = nullptr;
-
-struct DnsapiInfo {
- public:
-  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(DnsapiInfo);
-
-  HMODULE mLibrary;
-  decltype(&DnsQuery_A) mDnsQueryFunc;
-  decltype(&DnsFree) mDnsFreeFunc;
-
- private:
-  // This will either be called during shutdown of the GetAddrInfo module, or
-  // when a worker thread is done doing a lookup (ie: within
-  // _GetAddrInfo_Windows). Note that the lock must be held when this is
-  // called.
-  ~DnsapiInfo() {
-    if (gDnsapiInfoLock) {
-      gDnsapiInfoLock->AssertCurrentThreadOwns();
-    } else {
-      MOZ_ASSERT_UNREACHABLE(
-          "No mutex available during GetAddrInfo "
-          "shutdown.");
-      return;
-    }
-
-    LOG("Freeing Dnsapi.dll");
-    MOZ_ASSERT(mLibrary);
-    DebugOnly<BOOL> rv = FreeLibrary(mLibrary);
-    NS_WARNING_ASSERTION(rv, "Failed to free Dnsapi.dll.");
-  }
-};
-
-static StaticRefPtr<DnsapiInfo> gDnsapiInfo;
-
-static MOZ_ALWAYS_INLINE nsresult _GetAddrInfoInit_Windows() {
-  // This is necessary to ensure strict thread safety because if two threads
-  // run this function at the same time they can potentially create two
-  // mutexes.
-  MOZ_ASSERT(NS_IsMainThread(),
-             "Do not initialize GetAddrInfo off main thread!");
-
-  if (!gDnsapiInfoLock) {
-    gDnsapiInfoLock = new OffTheBooksMutex("GetAddrInfo.cpp::gDnsapiInfoLock");
-  }
-  OffTheBooksMutexAutoLock lock(*gDnsapiInfoLock);
-
-  if (gDnsapiInfo) {
-    MOZ_ASSERT_UNREACHABLE("GetAddrInfo is being initialized multiple times!");
-    return NS_ERROR_ALREADY_INITIALIZED;
-  }
-
-  HMODULE library = LoadLibraryA("Dnsapi.dll");
-  if (NS_WARN_IF(!library)) {
-    return NS_ERROR_FAILURE;
-  }
-
-  FARPROC dnsQueryFunc = GetProcAddress(library, "DnsQuery_A");
-  FARPROC dnsFreeFunc = GetProcAddress(library, "DnsFree");
-  if (NS_WARN_IF(!dnsQueryFunc) || NS_WARN_IF(!dnsFreeFunc)) {
-    DebugOnly<BOOL> rv = FreeLibrary(library);
-    NS_WARNING_ASSERTION(rv, "Failed to free Dnsapi.dll.");
-    return NS_ERROR_FAILURE;
-  }
-
-  DnsapiInfo* info = new DnsapiInfo;
-  info->mLibrary = library;
-  info->mDnsQueryFunc = (decltype(info->mDnsQueryFunc))dnsQueryFunc;
-  info->mDnsFreeFunc = (decltype(info->mDnsFreeFunc))dnsFreeFunc;
-  gDnsapiInfo = info;
-
-  return NS_OK;
-}
-
-static MOZ_ALWAYS_INLINE nsresult _GetAddrInfoShutdown_Windows() {
-  OffTheBooksMutexAutoLock lock(*gDnsapiInfoLock);
-
-  if (NS_WARN_IF(!gDnsapiInfo) || NS_WARN_IF(!gDnsapiInfoLock)) {
-    MOZ_ASSERT_UNREACHABLE("GetAddrInfo not initialized!");
-    return NS_ERROR_NOT_INITIALIZED;
-  }
-
-  gDnsapiInfo = nullptr;
-
-  return NS_OK;
-}
-
 // If successful, returns in aResult a TTL value that is smaller or
 // equal with the one already there. Gets the TTL value by calling
-// to dnsapi->mDnsQueryFunc and iterating through the returned
+// to DnsQuery_A and iterating through the returned
 // records to find the one with the smallest TTL value.
-static MOZ_ALWAYS_INLINE nsresult
-_GetMinTTLForRequestType_Windows(DnsapiInfo* dnsapi, const char* aHost,
-                                 uint16_t aRequestType, unsigned int* aResult) {
-  MOZ_ASSERT(dnsapi);
+static MOZ_ALWAYS_INLINE nsresult _GetMinTTLForRequestType_Windows(
+    const char* aHost, uint16_t aRequestType, unsigned int* aResult) {
   MOZ_ASSERT(aHost);
   MOZ_ASSERT(aResult);
 
   PDNS_RECORDA dnsData = nullptr;
-  DNS_STATUS status = dnsapi->mDnsQueryFunc(
+  DNS_STATUS status = DnsQuery_A(
       aHost, aRequestType,
       (DNS_QUERY_STANDARD | DNS_QUERY_NO_NETBT | DNS_QUERY_NO_HOSTS_FILE |
        DNS_QUERY_NO_MULTICAST | DNS_QUERY_ACCEPT_TRUNCATED_RESPONSE |
@@ -184,46 +91,29 @@ _GetMinTTLForRequestType_Windows(DnsapiInfo* dnsapi, const char* aHost,
     }
   }
 
-  dnsapi->mDnsFreeFunc(dnsData, DNS_FREE_TYPE::DnsFreeRecordList);
+  DnsFree(dnsData, DNS_FREE_TYPE::DnsFreeRecordList);
   return NS_OK;
 }
 
-static MOZ_ALWAYS_INLINE nsresult _GetTTLData_Windows(const char* aHost,
+static MOZ_ALWAYS_INLINE nsresult _GetTTLData_Windows(const nsACString& aHost,
                                                       uint32_t* aResult,
                                                       uint16_t aAddressFamily) {
-  MOZ_ASSERT(aHost);
+  MOZ_ASSERT(!aHost.IsEmpty());
   MOZ_ASSERT(aResult);
   if (aAddressFamily != PR_AF_UNSPEC && aAddressFamily != PR_AF_INET &&
       aAddressFamily != PR_AF_INET6) {
     return NS_ERROR_UNEXPECTED;
   }
 
-  RefPtr<DnsapiInfo> dnsapi = nullptr;
-  {
-    OffTheBooksMutexAutoLock lock(*gDnsapiInfoLock);
-    dnsapi = gDnsapiInfo;
-  }
-
-  if (!dnsapi) {
-    LOG_WARNING("GetAddrInfo has been shutdown or has not been initialized.");
-    return NS_ERROR_NOT_INITIALIZED;
-  }
-
   // In order to avoid using ANY records which are not always implemented as a
   // "Gimme what you have" request in hostname resolvers, we should send A
   // and/or AAAA requests, based on the address family requested.
   unsigned int ttl = (unsigned int)-1;
   if (aAddressFamily == PR_AF_UNSPEC || aAddressFamily == PR_AF_INET) {
-    _GetMinTTLForRequestType_Windows(dnsapi, aHost, DNS_TYPE_A, &ttl);
+    _GetMinTTLForRequestType_Windows(aHost.BeginReading(), DNS_TYPE_A, &ttl);
   }
   if (aAddressFamily == PR_AF_UNSPEC || aAddressFamily == PR_AF_INET6) {
-    _GetMinTTLForRequestType_Windows(dnsapi, aHost, DNS_TYPE_AAAA, &ttl);
-  }
-
-  {
-    // dnsapi's destructor is not thread-safe, so we release explicitly here
-    OffTheBooksMutexAutoLock lock(*gDnsapiInfoLock);
-    dnsapi = nullptr;
+    _GetMinTTLForRequestType_Windows(aHost.BeginReading(), DNS_TYPE_AAAA, &ttl);
   }
 
   if (ttl == (unsigned int)-1) {
@@ -240,10 +130,10 @@ static MOZ_ALWAYS_INLINE nsresult _GetTTLData_Windows(const char* aHost,
 // PORTABLE RUNTIME IMPLEMENTATION//
 ////////////////////////////////////
 
-static MOZ_ALWAYS_INLINE nsresult _GetAddrInfo_Portable(
-    const char* aCanonHost, uint16_t aAddressFamily, uint16_t aFlags,
-    const char* aNetworkInterface, AddrInfo** aAddrInfo) {
-  MOZ_ASSERT(aCanonHost);
+static MOZ_ALWAYS_INLINE nsresult
+_GetAddrInfo_Portable(const nsACString& aCanonHost, uint16_t aAddressFamily,
+                      uint16_t aFlags, AddrInfo** aAddrInfo) {
+  MOZ_ASSERT(!aCanonHost.IsEmpty());
   MOZ_ASSERT(aAddrInfo);
 
   // We accept the same aFlags that nsHostResolver::ResolveHost accepts, but we
@@ -261,27 +151,28 @@ static MOZ_ALWAYS_INLINE nsresult _GetAddrInfo_Portable(
     aAddressFamily = PR_AF_UNSPEC;
   }
 
-  PRAddrInfo* prai = PR_GetAddrInfoByName(aCanonHost, aAddressFamily, prFlags);
+  PRAddrInfo* prai =
+      PR_GetAddrInfoByName(aCanonHost.BeginReading(), aAddressFamily, prFlags);
 
   if (!prai) {
     return NS_ERROR_UNKNOWN_HOST;
   }
 
-  const char* canonName = nullptr;
+  nsAutoCString canonName;
   if (aFlags & nsHostResolver::RES_CANON_NAME) {
-    canonName = PR_GetCanonNameFromAddrInfo(prai);
+    canonName.Assign(PR_GetCanonNameFromAddrInfo(prai));
   }
 
   bool filterNameCollision =
       !(aFlags & nsHostResolver::RES_ALLOW_NAME_COLLISION);
-  nsAutoPtr<AddrInfo> ai(new AddrInfo(aCanonHost, prai, disableIPv4,
-                                      filterNameCollision, canonName));
+  RefPtr<AddrInfo> ai(new AddrInfo(aCanonHost, prai, disableIPv4,
+                                   filterNameCollision, canonName));
   PR_FreeAddrInfo(prai);
   if (ai->mAddresses.isEmpty()) {
     return NS_ERROR_UNKNOWN_HOST;
   }
 
-  *aAddrInfo = ai.forget();
+  ai.forget(aAddrInfo);
 
   return NS_OK;
 }
@@ -291,31 +182,59 @@ static MOZ_ALWAYS_INLINE nsresult _GetAddrInfo_Portable(
 //////////////////////////////////////
 nsresult GetAddrInfoInit() {
   LOG("Initializing GetAddrInfo.\n");
-
-#ifdef DNSQUERY_AVAILABLE
-  return _GetAddrInfoInit_Windows();
-#else
   return NS_OK;
-#endif
 }
 
 nsresult GetAddrInfoShutdown() {
   LOG("Shutting down GetAddrInfo.\n");
-
-#ifdef DNSQUERY_AVAILABLE
-  return _GetAddrInfoShutdown_Windows();
-#else
   return NS_OK;
-#endif
 }
 
-nsresult GetAddrInfo(const char* aHost, uint16_t aAddressFamily,
-                     uint16_t aFlags, const char* aNetworkInterface,
-                     AddrInfo** aAddrInfo, bool aGetTtl) {
-  if (NS_WARN_IF(!aHost) || NS_WARN_IF(!aAddrInfo)) {
+bool FindAddrOverride(const nsACString& aHost, uint16_t aAddressFamily,
+                      uint16_t aFlags, AddrInfo** aAddrInfo) {
+  RefPtr<NativeDNSResolverOverride> overrideService = gOverrideService;
+  if (!overrideService) {
+    return false;
+  }
+  AutoReadLock lock(overrideService->mLock);
+  nsTArray<PRNetAddr>* overrides = overrideService->mOverrides.GetValue(aHost);
+  if (!overrides) {
+    return false;
+  }
+  nsCString* cname = nullptr;
+  if (aFlags & nsHostResolver::RES_CANON_NAME) {
+    cname = overrideService->mCnames.GetValue(aHost);
+  }
+
+  RefPtr<AddrInfo> ai;
+
+  if (!cname) {
+    ai = new AddrInfo(aHost, 0);
+  } else {
+    ai = new AddrInfo(aHost, *cname, 0);
+  }
+
+  for (const auto& ip : *overrides) {
+    if (aAddressFamily != AF_UNSPEC && ip.raw.family != aAddressFamily) {
+      continue;
+    }
+    ai->AddAddress(new NetAddrElement(&ip));
+  }
+
+  ai.forget(aAddrInfo);
+  return true;
+}
+
+nsresult GetAddrInfo(const nsACString& aHost, uint16_t aAddressFamily,
+                     uint16_t aFlags, AddrInfo** aAddrInfo, bool aGetTtl) {
+  if (NS_WARN_IF(aHost.IsEmpty()) || NS_WARN_IF(!aAddrInfo)) {
     return NS_ERROR_NULL_POINTER;
   }
 
+  if (StaticPrefs::network_dns_disabled()) {
+    return NS_ERROR_UNKNOWN_HOST;
+  }
+
 #ifdef DNSQUERY_AVAILABLE
   // The GetTTLData needs the canonical name to function properly
   if (aGetTtl) {
@@ -323,35 +242,42 @@ nsresult GetAddrInfo(const char* aHost, uint16_t aAddressFamily,
   }
 #endif
 
+  // If there is an override for this host, then we synthetize a result.
+  if (gOverrideService &&
+      FindAddrOverride(aHost, aAddressFamily, aFlags, aAddrInfo)) {
+    return NS_OK;
+  }
+
+  nsAutoCString host(aHost);
   if (gNativeIsLocalhost) {
     // pretend we use the given host but use IPv4 localhost instead!
-    aHost = "localhost";
+    host = NS_LITERAL_CSTRING("localhost");
     aAddressFamily = PR_AF_INET;
   }
 
   *aAddrInfo = nullptr;
-  nsresult rv = _GetAddrInfo_Portable(aHost, aAddressFamily, aFlags,
-                                      aNetworkInterface, aAddrInfo);
+  nsresult rv = _GetAddrInfo_Portable(host, aAddressFamily, aFlags, aAddrInfo);
 
 #ifdef DNSQUERY_AVAILABLE
   if (aGetTtl && NS_SUCCEEDED(rv)) {
     // Figure out the canonical name, or if that fails, just use the host name
     // we have.
-    const char* name = nullptr;
-    if (*aAddrInfo != nullptr && (*aAddrInfo)->mCanonicalName) {
+    nsAutoCString name;
+    if (*aAddrInfo != nullptr && !(*aAddrInfo)->mCanonicalName.IsEmpty()) {
       name = (*aAddrInfo)->mCanonicalName;
     } else {
-      name = aHost;
+      name = host;
     }
 
-    LOG("Getting TTL for %s (cname = %s).", aHost, name);
+    LOG("Getting TTL for %s (cname = %s).", host.get(), name.get());
     uint32_t ttl = 0;
     nsresult ttlRv = _GetTTLData_Windows(name, &ttl, aAddressFamily);
     if (NS_SUCCEEDED(ttlRv)) {
       (*aAddrInfo)->ttl = ttl;
-      LOG("Got TTL %u for %s (name = %s).", ttl, aHost, name);
+      LOG("Got TTL %u for %s (name = %s).", ttl, host.get(), name.get());
     } else {
-      LOG_WARNING("Could not get TTL for %s (cname = %s).", aHost, name);
+      LOG_WARNING("Could not get TTL for %s (cname = %s).", host.get(),
+                  name.get());
     }
   }
 #endif
@@ -359,5 +285,70 @@ nsresult GetAddrInfo(const char* aHost, uint16_t aAddressFamily,
   return rv;
 }
 
+// static
+already_AddRefed<nsINativeDNSResolverOverride>
+NativeDNSResolverOverride::GetSingleton() {
+  if (gOverrideService) {
+    return do_AddRef(gOverrideService);
+  }
+
+  gOverrideService = new NativeDNSResolverOverride();
+  ClearOnShutdown(&gOverrideService);
+  return do_AddRef(gOverrideService);
+}
+
+NS_IMPL_ISUPPORTS(NativeDNSResolverOverride, nsINativeDNSResolverOverride)
+
+NS_IMETHODIMP NativeDNSResolverOverride::AddIPOverride(
+    const nsACString& aHost, const nsACString& aIPLiteral) {
+  PRNetAddr tempAddr;
+  // Unfortunately, PR_StringToNetAddr does not properly initialize
+  // the output buffer in the case of IPv6 input. See bug 223145.
+  memset(&tempAddr, 0, sizeof(PRNetAddr));
+
+  if (PR_StringToNetAddr(nsCString(aIPLiteral).get(), &tempAddr) !=
+      PR_SUCCESS) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  AutoWriteLock lock(mLock);
+  auto& overrides = mOverrides.GetOrInsert(aHost);
+  overrides.AppendElement(tempAddr);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP NativeDNSResolverOverride::SetCnameOverride(
+    const nsACString& aHost, const nsACString& aCNAME) {
+  if (aCNAME.IsEmpty()) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  AutoWriteLock lock(mLock);
+  mCnames.Put(aHost, nsCString(aCNAME));
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP NativeDNSResolverOverride::ClearHostOverride(
+    const nsACString& aHost) {
+  AutoWriteLock lock(mLock);
+  mCnames.Remove(aHost);
+  auto overrides = mOverrides.GetAndRemove(aHost);
+  if (!overrides) {
+    return NS_OK;
+  }
+
+  overrides->Clear();
+  return NS_OK;
+}
+
+NS_IMETHODIMP NativeDNSResolverOverride::ClearOverrides() {
+  AutoWriteLock lock(mLock);
+  mOverrides.Clear();
+  mCnames.Clear();
+  return NS_OK;
+}
+
 }  // namespace net
 }  // namespace mozilla
